# Copyright (C) 2013, Eric Wong <normalperson@yhbt.net> and all contributors
# License: GPLv3 or later (https://www.gnu.org/licenses/gpl-3.0.txt)
require_relative 'server_helper'

class TestResponse < Testcase
  ENV["N"].to_i > 1 and parallelize_me!
  include ServerHelper
  alias setup server_helper_setup
  alias teardown server_helper_teardown

  def test_response_time_empty_body
    err = @err
    cfg = Yahns::Config.new
    host, port = @srv.addr[3], @srv.addr[1]
    cfg.instance_eval do
      GTL.synchronize do
        app = Rack::Builder.new do
          use Rack::ContentLength
          use Rack::ContentType, "text/plain"
          run lambda { |_| [ 200, {}, [] ] }
        end
        app(:rack, app) do
          listen "#{host}:#{port}"
        end
      end
      logger(Logger.new(err.path))
    end
    pid = mkserver(cfg)
    Net::HTTP.start(host, port) { |h|
      # warmup request
      res = h.get("/")
      assert_empty res.body

      t0 = Time.now
      nr = 10
      nr.times do
        res = h.get("/")
        assert_empty res.body
      end
      diff = Time.now - t0
      assert_operator diff, :<, (0.200 * nr)
    }
  ensure
    quit_wait(pid)
  end

  def test_response_time_head
    err = @err
    cfg = Yahns::Config.new
    host, port = @srv.addr[3], @srv.addr[1]
    cfg.instance_eval do
      GTL.synchronize do
        require 'rack/lobster'
        app = Rack::Builder.new do
          use Rack::Head
          use Rack::ContentLength
          use Rack::ContentType, "text/plain"
          run Rack::Lobster.new
        end
        app(:rack, app) do
          listen "#{host}:#{port}"
        end
      end
      logger(Logger.new(err.path))
    end
    pid = mkserver(cfg)
    Net::HTTP.start(host, port) { |h|
      # warmup request
      res = h.head("/")
      assert_equal 200, res.code.to_i

      t0 = Time.now
      nr = 10
      nr.times do
        res = h.head("/")
        assert_equal 200, res.code.to_i
      end
      diff = Time.now - t0
      assert_operator diff, :<, (0.200 * nr)
    }
  ensure
    quit_wait(pid)
  end
end
